"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uppercamelcase = require("uppercamelcase");
const isNestedField = (field) => {
    return field.field.indexOf('.') !== -1;
};
const isArrayType = (type) => {
    return type.endsWith('[]');
};
/**
 * Creates `docs` for our sdk
 *
 * @param information - Returns the information in a correct way so vscode can interpret it.
 */
const generateDocs = (information) => {
    if (!information) {
        return '';
    }
    return `/**
		 	* ${information.replace('<p>', '').replace('</p>', '')}
		 	*/`;
};
/**
 * Returns one line of the textual representation of an interface.
 *
 * @example "name?: string"
 * @param field - The field of which to create the property
 * @param isAny - Flag to indicate if an object if of type `any`
 */
const createInterfaceProperty = (field, isAny) => {
    return `${generateDocs(field.description)}
	${field.field}${field.optional ? '?' : ''}: ${isAny || !field.type ? 'any' : field.type.toLowerCase().replace('/', '|')};`;
};
const createInterfaceProperties = (map, response) => {
    const properties = [];
    for (const [key, value] of Array.from(map.entries())) {
        if (response) {
            properties.push(value.join('\n'));
            continue;
        }
        const newParams = `
			${key}: {
				${value.join('\n')}
			};
		`;
        properties.push(newParams);
    }
    return properties.join('\n');
};
/**
 * Maps the field object to an array of properties as they should be desplayed in the interface
 *
 * @param fields - Property fields to loop over;
 */
exports.loopOverFields = (fields) => {
    const stringifiedFields = [];
    for (const field of fields) {
        const { type } = field;
        // If we are on a nested property, do nothing and go to the next field.
        if (isNestedField(field)) {
            continue;
        }
        if (!field.type) {
            return stringifiedFields;
        }
        // If the field is of type object
        if (type.toLowerCase().replace(/\[]/g, '') === 'object') {
            const name = field.field;
            const nestedProperties = [];
            // Go find all his nested properties and push them in the array
            for (const prop of fields) {
                if (!prop.field.startsWith(name) || !isNestedField(prop)) {
                    continue;
                }
                // Push the field into the object, strip off the parent property. 'parent.child.grandchild' => 'child.grandchild'
                const strippedProp = Object.assign(Object.assign({}, prop), { field: prop.field.split('.').slice(1).join('.') });
                // And push this object in the array
                nestedProperties.push(strippedProp);
            }
            // If the array of nested properties is empty, type it as `any`
            if (nestedProperties.length === 0) {
                stringifiedFields.push(createInterfaceProperty(field, true));
                continue;
            }
            // For every field in the array, loop back over this function
            stringifiedFields.push(`${generateDocs(field.description)}
				${name}${field.optional ? '?' : ''}: {
					${exports.loopOverFields(nestedProperties).join('\n')}
				}${isArrayType(type) ? '[]' : ''};`);
            continue;
        }
        stringifiedFields.push(createInterfaceProperty(field));
    }
    return stringifiedFields;
};
/**
 * Maps the properties of `params` `body` or `Success` to an array of their stringified field
 *
 * @param fields - The fields for which we wish to create a parammap
 */
const createParamMap = (fields) => {
    const paramMap = new Map();
    if (!fields) {
        return;
    }
    for (const objectName of Object.keys(fields)) {
        const properties = fields[objectName];
        const stringifiedFields = exports.loopOverFields(properties);
        const isOptional = !properties.filter(field => !isNestedField(field)).some(field => !field.optional);
        paramMap.set(`${objectName}${isOptional ? '?' : ''}`, stringifiedFields);
    }
    return paramMap;
};
/**
 * Generates both the Request and Response interface for one function.
 *
 * @param docs - Docs object generated by the API docs.
 */
exports.createInterface = (docs) => {
    const { name, parameter, success } = docs;
    const requestName = `${uppercamelcase(name)}Request`;
    const requestParamMap = createParamMap(parameter && parameter.fields);
    const hasSuccessField = success && success.fields;
    const successField = hasSuccessField && success.fields.Success && success.fields.Success[0] && success.fields.Success[0].field;
    const hasNoResponse = !hasSuccessField || successField === '204';
    const hasResponseOfTypeAny = successField === '200';
    let responseParamMap;
    let responseName;
    if (!hasNoResponse && !hasResponseOfTypeAny) {
        responseParamMap = createParamMap(success.fields);
        responseName = `${uppercamelcase(name)}Response`;
    }
    if (hasResponseOfTypeAny) {
        console.warn('API:', name, '\n\nThe API has a responsetype of `any`.\nTake more care of your documentation my young padawan.\n\n');
    }
    return `${!requestParamMap ? '' : `export interface ${requestName} {
		${createInterfaceProperties(requestParamMap)}
	}`}
	${!responseParamMap ? '' : `export interface ${responseName} {
		${createInterfaceProperties(responseParamMap, true)}
	}`}
	${generateDocs(docs.description)}
	export function ${name}(${!requestParamMap ? '' : `request: ${requestName}`}): Promise<${hasNoResponse ? 'void' : `${hasResponseOfTypeAny ? 'any' : responseName}`}>;
	`;
};
