import {loopOverFields, createInterface} from '../../src/util';
import {getDocs} from '../fixtures/get-docs';
import {getExpected} from '../fixtures/get-expected';

const cleanString = (s: string) => {
	return s.replace(/\s/g, '');
};

const cleanStringArray = (array: string[]): string[] => {
	return array.map(s => cleanString(s));
};

test('Loop over empty fields array should return empty array', () => {
	const result = loopOverFields([]);

	expect(result).toEqual([]);
});

test('should return correct array of string when non nested property', () => {
	const data = [
		{
			group: 'params',
			type: 'String',
			optional: false,
			field: 'organisation'
		},
		{
			group: 'params',
			type: 'String',
			optional: false,
			field: 'id'
		}
	];

	const result = loopOverFields(data);

	expect(cleanStringArray(result)).toEqual(['organisation:string;', 'id:string;']);
});

test('should return correct array of string when nested property', () => {
	const data = [
		{
			group: 'params',
			type: 'Object',
			optional: false,
			field: 'organisation'
		},
		{
			group: 'params',
			type: 'String',
			optional: false,
			field: 'organisation.id'
		},
		{
			group: 'params',
			type: 'String',
			optional: false,
			field: 'organisation.name'
		},
		{
			group: 'params',
			type: 'Number',
			optional: false,
			field: 'amount'
		}
	];

	const result = loopOverFields(data);

	expect(cleanStringArray(result)).toEqual(['organisation:{id:string;name:string;};', 'amount:number;']);
});

test('should return correct string when an array type', () => {
	const data: any[] = [
		{
			group: 'params',
			type: 'String[]',
			optional: false,
			field: 'organisation'
		}
	];

	const result = loopOverFields(data);

	expect(cleanStringArray(result)).toEqual(['organisation:string[];']);
});

test('should return correct string when an array object type', () => {
	const data: any[] = [
		{
			group: 'params',
			type: 'Object[]',
			optional: false,
			field: 'items'
		},
		{
			group: 'params',
			type: 'String',
			optional: false,
			field: 'items.id'
		},
		{
			group: 'params',
			type: 'String',
			optional: false,
			field: 'items.name'
		},
		{
			group: 'params',
			type: 'Number',
			optional: false,
			field: 'amount'
		}
	];

	const result = loopOverFields(data);

	expect(cleanStringArray(result)).toEqual(['items:{id:string;name:string;}[];', 'amount:number;']);
});

test('should return string with docs', () => {
	const data: any[] = [
		{
			group: 'params',
			type: 'Object[]',
			optional: false,
			field: 'items',
			description: 'All the item'
		},
		{
			group: 'params',
			type: 'String',
			optional: false,
			field: 'items.id'
		},
		{
			group: 'params',
			type: 'String',
			optional: false,
			field: 'items.name'
		},
		{
			group: 'params',
			type: 'Number',
			optional: false,
			field: 'amount'
		}
	];

	const result = loopOverFields(data);

	expect(cleanStringArray(result)).toEqual(['/***Alltheitem*/items:{id:string;name:string;}[];', 'amount:number;']);
});

test('create interface for normal values', async () => {
	const docs = await getDocs('create-basic-interface.json');
	const expected = getExpected('create-basic-interface.d.ts');

	const result = docs.map(doc => createInterface(doc));

	expect(cleanString(result.join(''))).toBe(cleanString(expected));
});

test('Interfaces with no return values', async () => {
	const docs = await getDocs('create-interface-no-return.json');
	const expected = getExpected('create-interface-no-return.d.ts');

	const result = docs.map(doc => createInterface(doc));

	expect(cleanString(result.join(''))).toBe(cleanString(expected));
});

test('Interfaces with no params', async () => {
	const docs = await getDocs('create-interface-no-params.json');
	const expected = getExpected('create-interface-no-params.d.ts');

	const result = docs.map(doc => createInterface(doc));

	console.log(JSON.stringify(result, undefined, '\t'));

	expect(cleanString(result.join(''))).toBe(cleanString(expected));
});
